@@ -107,13 +107,28 @@ fn bench_eval_call(bench: &mut Bencher) {
 }
 
 #[bench]
-fn bench_eval_loop_number(bench: &mut Bencher) {
+fn bench_eval_deeply_nested(bench: &mut Bencher) {
     let script = r#"
+            (1 + 2 * 3 - 9) * 4 < 5 * 6 - 70 / 8 &&
+            (42 + 99 > 1 + 2 - 3 + 4 * 5 || 123 - 88 < 123 + 88 - 99 + 100)
+            && true
+            && !!!!!!!!false
+        "#;
+
+    let mut engine = Engine::new();
+    engine.set_optimization_level(OptimizationLevel::None);
+
+    bench.iter(|| engine.eval::<bool>(script).unwrap());
+}
+
+#[bench]
+fn bench_eval_loop_number(bench: &mut Bencher) {
+    let script = "
         let s = 0;
-        for x in range(0, 10000) {
+        for x in 0..10000 {
             s += 1;
         }
-    "#;
+    ";
 
     let mut engine = Engine::new();
     engine.set_optimization_level(OptimizationLevel::None);
@@ -127,7 +142,7 @@ fn bench_eval_loop_number(bench: &mut Bencher) {
 fn bench_eval_loop_strings_build(bench: &mut Bencher) {
     let script = r#"
         let s;
-        for x in range(0, 10000) {
+        for x in 0..10000 {
             s = "hello, world!" + "hello, world!";
         }
     "#;
@@ -144,7 +159,7 @@ fn bench_eval_loop_strings_build(bench: &mut Bencher) {
 fn bench_eval_loop_strings_no_build(bench: &mut Bencher) {
     let script = r#"
         let s;
-        for x in range(0, 10000) {
+        for x in 0..10000 {
             s = "hello" + "";
         }
     "#;
@@ -159,11 +174,11 @@ fn bench_eval_loop_strings_no_build(bench: &mut Bencher) {
 
 #[bench]
 fn bench_eval_switch(bench: &mut Bencher) {
-    let script = r#"
+    let script = "
         let sum = 0;
         let rem = 0;
 
-        for x in range(0, 10) {
+        for x in 0..10 {
             rem = x % 10;
 
             sum += switch rem {
@@ -179,7 +194,7 @@ fn bench_eval_switch(bench: &mut Bencher) {
                 9 => 1,
             }
         }
-    "#;
+    ";
 
     let mut engine = Engine::new();
     engine.set_optimization_level(OptimizationLevel::None);
@@ -191,11 +206,11 @@ fn bench_eval_switch(bench: &mut Bencher) {
 
 #[bench]
 fn bench_eval_nested_if(bench: &mut Bencher) {
-    let script = r#"
+    let script = "
         let sum = 0;
         let rem = 0;
 
-        for x in range(0, 10) {
+        for x in 0..10 {
             rem = x % 10;
 
             sum += if rem == 0 { 10 }
@@ -209,7 +224,7 @@ fn bench_eval_nested_if(bench: &mut Bencher) {
             else if rem == 8 { 42 }
             else if rem == 9 { 1 };
         }
-    "#;
+    ";
 
     let mut engine = Engine::new();
     engine.set_optimization_level(OptimizationLevel::None);
