@@ -1,8 +1,11 @@
 #![cfg(not(feature = "no_std"))]
-#![cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
+#![cfg(not(target_arch = "wasm32"))]
+#![cfg(not(target_arch = "wasm64"))]
 
 use crate::func::native::shared_write_lock;
-use crate::{Engine, EvalAltResult, Identifier, Module, ModuleResolver, Position, Scope, Shared};
+use crate::{
+    Engine, Identifier, Module, ModuleResolver, Position, RhaiResultOf, Scope, Shared, ERR,
+};
 
 use std::{
     collections::BTreeMap,
@@ -223,7 +226,7 @@ impl FileModuleResolver {
         path: impl AsRef<str>,
         source_path: Option<impl AsRef<str>>,
     ) -> Option<Shared<Module>> {
-        let file_path = self.get_file_path(path.as_ref(), source_path.as_ref().map(|v| v.as_ref()));
+        let file_path = self.get_file_path(path.as_ref(), source_path.as_ref().map(<_>::as_ref));
 
         shared_write_lock(&self.cache)
             .remove_entry(&file_path)
@@ -259,7 +262,7 @@ impl ModuleResolver for FileModuleResolver {
         source_path: Option<&str>,
         path: &str,
         pos: Position,
-    ) -> Result<Shared<Module>, Box<EvalAltResult>> {
+    ) -> RhaiResultOf<Shared<Module>> {
         // Load relative paths from source if there is no base path specified
         let source_path =
             source_path.and_then(|p| Path::new(p).parent().map(|p| p.to_string_lossy()));
@@ -285,17 +288,17 @@ impl ModuleResolver for FileModuleResolver {
         let mut ast = engine
             .compile_file(file_path.clone())
             .map_err(|err| match *err {
-                EvalAltResult::ErrorSystem(_, err) if err.is::<IoError>() => {
-                    Box::new(EvalAltResult::ErrorModuleNotFound(path.to_string(), pos))
+                ERR::ErrorSystem(_, err) if err.is::<IoError>() => {
+                    Box::new(ERR::ErrorModuleNotFound(path.to_string(), pos))
                 }
-                _ => Box::new(EvalAltResult::ErrorInModule(path.to_string(), err, pos)),
+                _ => Box::new(ERR::ErrorInModule(path.to_string(), err, pos)),
             })?;
 
         ast.set_source(path);
 
         // Make a module from the AST
         let m: Shared<Module> = Module::eval_ast_as_new(scope, &ast, engine)
-            .map_err(|err| Box::new(EvalAltResult::ErrorInModule(path.to_string(), err, pos)))?
+            .map_err(|err| Box::new(ERR::ErrorInModule(path.to_string(), err, pos)))?
             .into();
 
         // Put it into the cache
@@ -315,7 +318,7 @@ impl ModuleResolver for FileModuleResolver {
         source_path: Option<&str>,
         path: &str,
         pos: Position,
-    ) -> Option<Result<crate::AST, Box<EvalAltResult>>> {
+    ) -> Option<RhaiResultOf<crate::AST>> {
         // Construct the script file path
         let file_path = self.get_file_path(path, source_path);
 
@@ -328,10 +331,10 @@ impl ModuleResolver for FileModuleResolver {
                     ast
                 })
                 .map_err(|err| match *err {
-                    EvalAltResult::ErrorSystem(_, err) if err.is::<IoError>() => {
-                        EvalAltResult::ErrorModuleNotFound(path.to_string(), pos).into()
+                    ERR::ErrorSystem(_, err) if err.is::<IoError>() => {
+                        ERR::ErrorModuleNotFound(path.to_string(), pos).into()
                     }
-                    _ => EvalAltResult::ErrorInModule(path.to_string(), err, pos).into(),
+                    _ => ERR::ErrorInModule(path.to_string(), err, pos).into(),
                 }),
         )
     }
