@@ -1,23 +1,19 @@
-use crate::fn_native::SendSync;
-use crate::{Engine, EvalAltResult, Module, Position, Shared, AST};
+use crate::func::native::SendSync;
+use crate::{Engine, Module, Position, RhaiResultOf, Shared, AST};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 
-mod dummy;
-pub use dummy::DummyModuleResolver;
-
 mod collection;
-pub use collection::ModuleResolversCollection;
-
-#[cfg(not(feature = "no_std"))]
-#[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
+mod dummy;
 mod file;
-
-#[cfg(not(feature = "no_std"))]
-#[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
-pub use file::FileModuleResolver;
-
 mod stat;
+
+pub use collection::ModuleResolversCollection;
+pub use dummy::DummyModuleResolver;
+#[cfg(not(feature = "no_std"))]
+#[cfg(not(target_arch = "wasm32"))]
+#[cfg(not(target_arch = "wasm64"))]
+pub use file::FileModuleResolver;
 pub use stat::StaticModuleResolver;
 
 /// Trait that encapsulates a module resolution service.
@@ -29,14 +25,14 @@ pub trait ModuleResolver: SendSync {
         source_path: Option<&str>,
         path: &str,
         pos: Position,
-    ) -> Result<Shared<Module>, Box<EvalAltResult>>;
+    ) -> RhaiResultOf<Shared<Module>>;
 
     /// Resolve an `AST` based on a path string.
     ///
     /// Returns [`None`] (default) if such resolution is not supported
     /// (e.g. if the module is Rust-based).
     ///
-    /// ## Low-Level API
+    /// # WARNING - Low Level API
     ///
     /// Override the default implementation of this method if the module resolver
     /// serves modules based on compiled Rhai scripts.
@@ -48,7 +44,7 @@ pub trait ModuleResolver: SendSync {
         source_path: Option<&str>,
         path: &str,
         pos: Position,
-    ) -> Option<Result<AST, Box<EvalAltResult>>> {
+    ) -> Option<RhaiResultOf<AST>> {
         None
     }
 }
