@@ -2,7 +2,10 @@
 #![allow(non_snake_case)]
 
 use crate::plugin::*;
-use crate::{def_package, Blob, Dynamic, EvalAltResult, NativeCallContext, Position, INT};
+use crate::{
+    def_package, Blob, Dynamic, ExclusiveRange, InclusiveRange, NativeCallContext, Position,
+    RhaiResultOf, INT,
+};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 use std::{any::TypeId, mem};
@@ -10,25 +13,25 @@ use std::{any::TypeId, mem};
 #[cfg(not(feature = "no_float"))]
 use crate::FLOAT;
 
-def_package!(crate:BasicBlobPackage:"Basic BLOB utilities.", lib, {
-    lib.standard = true;
+def_package! {
+    /// Package of basic BLOB utilities.
+    crate::BasicBlobPackage => |lib| {
+        lib.standard = true;
 
-    combine_with_exported_module!(lib, "blob", blob_functions);
+        combine_with_exported_module!(lib, "blob", blob_functions);
 
-    // Register blob iterator
-    lib.set_iterable::<Blob>();
-});
+        // Register blob iterator
+        lib.set_iterable::<Blob>();
+    }
+}
 
 #[export_module]
-mod blob_functions {
-    pub fn blob() -> Blob {
+pub mod blob_functions {
+    pub const fn blob() -> Blob {
         Blob::new()
     }
     #[rhai_fn(name = "blob", return_raw)]
-    pub fn blob_with_capacity(
-        ctx: NativeCallContext,
-        len: INT,
-    ) -> Result<Blob, Box<EvalAltResult>> {
+    pub fn blob_with_capacity(ctx: NativeCallContext, len: INT) -> RhaiResultOf<Blob> {
         blob_with_capacity_and_value(ctx, len, 0)
     }
     #[rhai_fn(name = "blob", return_raw)]
@@ -36,34 +39,30 @@ mod blob_functions {
         ctx: NativeCallContext,
         len: INT,
         value: INT,
-    ) -> Result<Blob, Box<EvalAltResult>> {
+    ) -> RhaiResultOf<Blob> {
         let len = if len < 0 { 0 } else { len as usize };
         let _ctx = ctx;
 
         // Check if blob will be over max size limit
         #[cfg(not(feature = "unchecked"))]
         if _ctx.engine().max_array_size() > 0 && len > _ctx.engine().max_array_size() {
-            return Err(EvalAltResult::ErrorDataTooLarge(
-                "Size of BLOB".to_string(),
-                Position::NONE,
-            )
-            .into());
+            return Err(
+                crate::ERR::ErrorDataTooLarge("Size of BLOB".to_string(), Position::NONE).into(),
+            );
         }
 
         let mut blob = Blob::new();
-        blob.resize(len, (value & 0x00ff) as u8);
+        blob.resize(len, (value & 0x000000ff) as u8);
         Ok(blob)
     }
     #[rhai_fn(name = "len", get = "len", pure)]
     pub fn len(blob: &mut Blob) -> INT {
         blob.len() as INT
     }
-    #[rhai_fn(name = "push", name = "+=")]
     pub fn push(blob: &mut Blob, item: INT) {
-        let item = (item & 0x00ff) as u8;
+        let item = (item & 0x000000ff) as u8;
         blob.push(item);
     }
-    #[rhai_fn(name = "append", name = "+=")]
     pub fn append(blob: &mut Blob, y: Blob) {
         if !y.is_empty() {
             if blob.is_empty() {
@@ -74,18 +73,21 @@ mod blob_functions {
         }
     }
     #[rhai_fn(name = "+")]
-    pub fn concat(mut blob: Blob, y: Blob) -> Blob {
-        if !y.is_empty() {
-            if blob.is_empty() {
-                blob = y;
+    pub fn concat(blob1: Blob, blob2: Blob) -> Blob {
+        if !blob2.is_empty() {
+            if blob1.is_empty() {
+                blob2
             } else {
-                blob.extend(y);
+                let mut blob = blob1;
+                blob.extend(blob2);
+                blob
             }
+        } else {
+            blob1
         }
-        blob
     }
     pub fn insert(blob: &mut Blob, position: INT, item: INT) {
-        let item = (item & 0x00ff) as u8;
+        let item = (item & 0x000000ff) as u8;
 
         if blob.is_empty() {
             blob.push(item);
@@ -106,27 +108,20 @@ mod blob_functions {
         }
     }
     #[rhai_fn(return_raw)]
-    pub fn pad(
-        ctx: NativeCallContext,
-        blob: &mut Blob,
-        len: INT,
-        item: INT,
-    ) -> Result<(), Box<EvalAltResult>> {
+    pub fn pad(ctx: NativeCallContext, blob: &mut Blob, len: INT, item: INT) -> RhaiResultOf<()> {
         if len <= 0 {
             return Ok(());
         }
 
-        let item = (item & 0x00ff) as u8;
+        let item = (item & 0x000000ff) as u8;
         let _ctx = ctx;
 
         // Check if blob will be over max size limit
         #[cfg(not(feature = "unchecked"))]
         if _ctx.engine().max_array_size() > 0 && (len as usize) > _ctx.engine().max_array_size() {
-            return Err(EvalAltResult::ErrorDataTooLarge(
-                "Size of BLOB".to_string(),
-                Position::NONE,
-            )
-            .into());
+            return Err(
+                crate::ERR::ErrorDataTooLarge("Size of BLOB".to_string(), Position::NONE).into(),
+            );
         }
 
         if len as usize > blob.len() {
@@ -184,6 +179,18 @@ mod blob_functions {
             blob.reverse();
         }
     }
+    #[rhai_fn(name = "splice")]
+    pub fn splice_range(blob: &mut Blob, range: ExclusiveRange, replace: Blob) {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        splice(blob, start, end - start, replace)
+    }
+    #[rhai_fn(name = "splice")]
+    pub fn splice_range_inclusive(blob: &mut Blob, range: InclusiveRange, replace: Blob) {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        splice(blob, start, end - start + 1, replace)
+    }
     pub fn splice(blob: &mut Blob, start: INT, len: INT, replace: Blob) {
         if blob.is_empty() {
             *blob = replace;
@@ -194,9 +201,9 @@ mod blob_functions {
         let start = if start < 0 {
             start
                 .checked_abs()
-                .map_or(0, |n| blob_len - (n as usize).min(blob_len))
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
         } else if start as usize >= blob_len {
-            blob.extend(replace.into_iter());
+            blob.extend(replace);
             return;
         } else {
             start as usize
@@ -212,6 +219,18 @@ mod blob_functions {
 
         blob.splice(start..start + len, replace.into_iter());
     }
+    #[rhai_fn(name = "extract")]
+    pub fn extract_range(blob: &mut Blob, range: ExclusiveRange) -> Blob {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        extract(blob, start, end - start)
+    }
+    #[rhai_fn(name = "extract")]
+    pub fn extract_range_inclusive(blob: &mut Blob, range: InclusiveRange) -> Blob {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        extract(blob, start, end - start + 1)
+    }
     pub fn extract(blob: &mut Blob, start: INT, len: INT) -> Blob {
         if blob.is_empty() || len <= 0 {
             return Blob::new();
@@ -221,7 +240,7 @@ mod blob_functions {
         let start = if start < 0 {
             start
                 .checked_abs()
-                .map_or(0, |n| blob_len - (n as usize).min(blob_len))
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
         } else if start as usize >= blob_len {
             return Blob::new();
         } else {
@@ -264,6 +283,18 @@ mod blob_functions {
             result
         }
     }
+    #[rhai_fn(name = "drain")]
+    pub fn drain_range(blob: &mut Blob, range: ExclusiveRange) -> Blob {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        drain(blob, start, end - start)
+    }
+    #[rhai_fn(name = "drain")]
+    pub fn drain_range_inclusive(blob: &mut Blob, range: InclusiveRange) -> Blob {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        drain(blob, start, end - start + 1)
+    }
     pub fn drain(blob: &mut Blob, start: INT, len: INT) -> Blob {
         if blob.is_empty() || len <= 0 {
             return Blob::new();
@@ -273,7 +304,7 @@ mod blob_functions {
         let start = if start < 0 {
             start
                 .checked_abs()
-                .map_or(0, |n| blob_len - (n as usize).min(blob_len))
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
         } else if start as usize >= blob_len {
             return Blob::new();
         } else {
@@ -288,6 +319,18 @@ mod blob_functions {
 
         blob.drain(start..start + len).collect()
     }
+    #[rhai_fn(name = "retain")]
+    pub fn retain_range(blob: &mut Blob, range: ExclusiveRange) -> Blob {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        retain(blob, start, end - start)
+    }
+    #[rhai_fn(name = "retain")]
+    pub fn retain_range_inclusive(blob: &mut Blob, range: InclusiveRange) -> Blob {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        retain(blob, start, end - start + 1)
+    }
     pub fn retain(blob: &mut Blob, start: INT, len: INT) -> Blob {
         if blob.is_empty() || len <= 0 {
             return Blob::new();
@@ -297,7 +340,7 @@ mod blob_functions {
         let start = if start < 0 {
             start
                 .checked_abs()
-                .map_or(0, |n| blob_len - (n as usize).min(blob_len))
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
         } else if start as usize >= blob_len {
             return mem::take(blob);
         } else {
@@ -315,26 +358,6 @@ mod blob_functions {
 
         drained
     }
-    pub fn contains(blob: &mut Blob, value: Dynamic) -> bool {
-        if blob.is_empty() {
-            return false;
-        }
-
-        let value = match value.as_int() {
-            Ok(value) => value as u8,
-            _ => return false,
-        };
-
-        blob.contains(&value)
-    }
-    #[rhai_fn(name = "==", pure)]
-    pub fn equals(blob1: &mut Blob, blob2: Blob) -> bool {
-        &*blob1 == &blob2
-    }
-    #[rhai_fn(name = "!=", pure)]
-    pub fn not_equals(blob1: &mut Blob, blob2: Blob) -> bool {
-        &*blob1 != &blob2
-    }
 
     #[inline]
     fn parse_int(blob: &mut Blob, start: INT, len: INT, is_le: bool) -> INT {
@@ -346,7 +369,7 @@ mod blob_functions {
         let start = if start < 0 {
             start
                 .checked_abs()
-                .map_or(0, |n| blob_len - (n as usize).min(blob_len))
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
         } else if start as usize >= blob_len {
             return 0;
         } else {
@@ -374,9 +397,33 @@ mod blob_functions {
         }
     }
 
+    #[rhai_fn(name = "parse_le_int")]
+    pub fn parse_le_int_range(blob: &mut Blob, range: ExclusiveRange) -> INT {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        parse_le_int(blob, start, end - start)
+    }
+    #[rhai_fn(name = "parse_le_int")]
+    pub fn parse_le_int_range_inclusive(blob: &mut Blob, range: InclusiveRange) -> INT {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        parse_le_int(blob, start, end - start + 1)
+    }
     pub fn parse_le_int(blob: &mut Blob, start: INT, len: INT) -> INT {
         parse_int(blob, start, len, true)
     }
+    #[rhai_fn(name = "parse_be_int")]
+    pub fn parse_be_int_range(blob: &mut Blob, range: ExclusiveRange) -> INT {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        parse_be_int(blob, start, end - start)
+    }
+    #[rhai_fn(name = "parse_be_int")]
+    pub fn parse_be_int_range_inclusive(blob: &mut Blob, range: InclusiveRange) -> INT {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        parse_be_int(blob, start, end - start + 1)
+    }
     pub fn parse_be_int(blob: &mut Blob, start: INT, len: INT) -> INT {
         parse_int(blob, start, len, false)
     }
@@ -391,7 +438,7 @@ mod blob_functions {
         let start = if start < 0 {
             start
                 .checked_abs()
-                .map_or(0, |n| blob_len - (n as usize).min(blob_len))
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
         } else if start as usize >= blob_len {
             return;
         } else {
@@ -408,21 +455,43 @@ mod blob_functions {
 
         let len = usize::min(len, INT_BYTES);
 
-        let mut buf = [0_u8; INT_BYTES];
-
-        buf.copy_from_slice(&if is_le {
+        let buf = if is_le {
             value.to_le_bytes()
         } else {
             value.to_be_bytes()
-        });
+        };
 
         blob[start..][..len].copy_from_slice(&buf[..len]);
     }
+    #[rhai_fn(name = "write_le_int")]
+    pub fn write_le_int_range(blob: &mut Blob, range: ExclusiveRange, value: INT) {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        write_le_int(blob, start, end - start, value)
+    }
+    #[rhai_fn(name = "write_le_int")]
+    pub fn write_le_int_range_inclusive(blob: &mut Blob, range: InclusiveRange, value: INT) {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        write_le_int(blob, start, end - start + 1, value)
+    }
     #[rhai_fn(name = "write_le")]
     pub fn write_le_int(blob: &mut Blob, start: INT, len: INT, value: INT) {
         write_int(blob, start, len, value, true)
     }
     #[rhai_fn(name = "write_be")]
+    pub fn write_be_int_range(blob: &mut Blob, range: ExclusiveRange, value: INT) {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        write_be_int(blob, start, end - start, value)
+    }
+    #[rhai_fn(name = "write_be")]
+    pub fn write_be_int_range_inclusive(blob: &mut Blob, range: InclusiveRange, value: INT) {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        write_be_int(blob, start, end - start + 1, value)
+    }
+    #[rhai_fn(name = "write_be")]
     pub fn write_be_int(blob: &mut Blob, start: INT, len: INT, value: INT) {
         write_int(blob, start, len, value, false)
     }
@@ -438,7 +507,7 @@ mod blob_functions {
         let start = if start < 0 {
             start
                 .checked_abs()
-                .map_or(0, |n| blob_len - (n as usize).min(blob_len))
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
         } else if start as usize >= blob_len {
             return 0.0;
         } else {
@@ -466,11 +535,39 @@ mod blob_functions {
         }
     }
 
+    #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "parse_le_float")]
+    pub fn parse_le_float_range(blob: &mut Blob, range: ExclusiveRange) -> FLOAT {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        parse_le_float(blob, start, end - start)
+    }
+    #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "parse_le_float")]
+    pub fn parse_le_float_range_inclusive(blob: &mut Blob, range: InclusiveRange) -> FLOAT {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        parse_le_float(blob, start, end - start + 1)
+    }
     #[cfg(not(feature = "no_float"))]
     pub fn parse_le_float(blob: &mut Blob, start: INT, len: INT) -> FLOAT {
         parse_float(blob, start, len, true)
     }
     #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "parse_be_float")]
+    pub fn parse_be_float_range(blob: &mut Blob, range: ExclusiveRange) -> FLOAT {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        parse_be_float(blob, start, end - start)
+    }
+    #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "parse_be_float")]
+    pub fn parse_be_float_range_inclusive(blob: &mut Blob, range: InclusiveRange) -> FLOAT {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        parse_be_float(blob, start, end - start + 1)
+    }
+    #[cfg(not(feature = "no_float"))]
     pub fn parse_be_float(blob: &mut Blob, start: INT, len: INT) -> FLOAT {
         parse_float(blob, start, len, false)
     }
@@ -486,7 +583,7 @@ mod blob_functions {
         let start = if start < 0 {
             start
                 .checked_abs()
-                .map_or(0, |n| blob_len - (n as usize).min(blob_len))
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
         } else if start as usize >= blob_len {
             return;
         } else {
@@ -502,25 +599,123 @@ mod blob_functions {
         const FLOAT_BYTES: usize = mem::size_of::<FLOAT>();
 
         let len = usize::min(len, FLOAT_BYTES);
-
-        let mut buf = [0_u8; FLOAT_BYTES];
-
-        buf.copy_from_slice(&if is_le {
+        let buf = if is_le {
             value.to_le_bytes()
         } else {
             value.to_be_bytes()
-        });
+        };
 
         blob[start..][..len].copy_from_slice(&buf[..len]);
     }
     #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "write_le_float")]
+    pub fn write_le_float_range(blob: &mut Blob, range: ExclusiveRange, value: FLOAT) {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        write_le_float(blob, start, end - start, value)
+    }
+    #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "write_le_float")]
+    pub fn write_le_float_range_inclusive(blob: &mut Blob, range: InclusiveRange, value: FLOAT) {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        write_le_float(blob, start, end - start + 1, value)
+    }
+    #[cfg(not(feature = "no_float"))]
     #[rhai_fn(name = "write_le")]
     pub fn write_le_float(blob: &mut Blob, start: INT, len: INT, value: FLOAT) {
         write_float(blob, start, len, value, true)
     }
     #[cfg(not(feature = "no_float"))]
     #[rhai_fn(name = "write_be")]
+    pub fn write_be_float_range(blob: &mut Blob, range: ExclusiveRange, value: FLOAT) {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        write_be_float(blob, start, end - start, value)
+    }
+    #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "write_be")]
+    pub fn write_be_float_range_inclusive(blob: &mut Blob, range: InclusiveRange, value: FLOAT) {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        write_be_float(blob, start, end - start + 1, value)
+    }
+    #[cfg(not(feature = "no_float"))]
+    #[rhai_fn(name = "write_be")]
     pub fn write_be_float(blob: &mut Blob, start: INT, len: INT, value: FLOAT) {
         write_float(blob, start, len, value, false)
     }
+    #[inline]
+    fn write_string(blob: &mut Blob, start: INT, len: INT, string: &str, ascii_only: bool) {
+        if len <= 0 || blob.is_empty() || string.is_empty() {
+            return;
+        }
+        let blob_len = blob.len();
+
+        let start = if start < 0 {
+            start
+                .checked_abs()
+                .map_or(0, |n| blob_len - usize::min(n as usize, blob_len))
+        } else if start as usize >= blob_len {
+            return;
+        } else {
+            start as usize
+        };
+
+        let len = if len as usize > blob_len - start {
+            blob_len - start
+        } else {
+            len as usize
+        };
+
+        let len = usize::min(len, string.len());
+
+        if ascii_only {
+            string
+                .chars()
+                .filter(char::is_ascii)
+                .take(len)
+                .map(|ch| ch as u8)
+                .enumerate()
+                .for_each(|(i, x)| blob[start + i] = x);
+        } else {
+            blob[start..][..len].copy_from_slice(&string.as_bytes()[..len]);
+        }
+    }
+    #[rhai_fn(name = "write_utf8")]
+    pub fn write_utf8_string(blob: &mut Blob, start: INT, len: INT, string: &str) {
+        write_string(blob, start, len, string, false)
+    }
+    #[rhai_fn(name = "write_utf8")]
+    pub fn write_utf8_string_range(blob: &mut Blob, range: ExclusiveRange, string: &str) {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        write_string(blob, start, end - start, string, false)
+    }
+    #[rhai_fn(name = "write_utf8")]
+    pub fn write_utf8_string_range_inclusive(blob: &mut Blob, range: InclusiveRange, string: &str) {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        write_string(blob, start, end - start + 1, string, false)
+    }
+    #[rhai_fn(name = "write_ascii")]
+    pub fn write_ascii_string(blob: &mut Blob, start: INT, len: INT, string: &str) {
+        write_string(blob, start, len, string, true)
+    }
+    #[rhai_fn(name = "write_ascii")]
+    pub fn write_ascii_string_range(blob: &mut Blob, range: ExclusiveRange, string: &str) {
+        let start = INT::max(range.start, 0);
+        let end = INT::max(range.end, start);
+        write_string(blob, start, end - start, string, true)
+    }
+    #[rhai_fn(name = "write_ascii")]
+    pub fn write_ascii_string_range_inclusive(
+        blob: &mut Blob,
+        range: InclusiveRange,
+        string: &str,
+    ) {
+        let start = INT::max(*range.start(), 0);
+        let end = INT::max(*range.end(), start);
+        write_string(blob, start, end - start + 1, string, true)
+    }
 }
