@@ -4,6 +4,7 @@ pub mod dynamic;
 pub mod error;
 pub mod fn_ptr;
 pub mod immutable_string;
+pub mod interner;
 pub mod parse_error;
 pub mod scope;
 
@@ -11,5 +12,6 @@ pub use dynamic::Dynamic;
 pub use error::EvalAltResult;
 pub use fn_ptr::FnPtr;
 pub use immutable_string::ImmutableString;
+pub use interner::StringsInterner;
 pub use parse_error::{LexError, ParseError, ParseErrorType};
 pub use scope::Scope;
