@@ -12,6 +12,7 @@ use rhai::Array;
 use rhai::Map;
 #[cfg(not(feature = "no_float"))]
 use rhai::FLOAT;
+#[cfg(not(feature = "no_float"))]
 #[cfg(feature = "decimal")]
 use rust_decimal::Decimal;
 
@@ -784,7 +785,7 @@ fn test_serde_blob() -> Result<(), Box<EvalAltResult>> {
     let r = engine.eval::<Dynamic>(
         "
             let x = blob(10);
-            for i in range(0, 10) { x[i] = i; }
+            for i in 0..10 { x[i] = i; }
             x
         ",
     )?;
